@@ -4,8 +4,8 @@ import pandas as pd
 import torch
 from torch import nn
 import matplotlib.pyplot as plt
-device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")
-
+import pickle
+from IPython import display
 
 
 from . import src
@@ -13,7 +13,6 @@ from . import models
 from . import generate_residuals
 from . import stastics
 
-
 class DL_AD():
     """        
     Пайплайн Time Series Anomaly Detection based on 
@@ -63,7 +62,8 @@ class DL_AD():
     --------
     https://github.com/waico/tsad/tree/main/examples 
     """
-    def _get_Train_Test_sets(self,dfs,len_seq,
+
+    def _get_Train_Test_sets(self, dfs, len_seq,
                              points_ahead,
                              gap,
                              shag,
@@ -71,79 +71,78 @@ class DL_AD():
                              test_size,
                              train_size,
                              random_state,
-                             shuffle,  
+                             shuffle,
                              stratify,
-                            ):
+                             ):
         """
         Вспомогательная функция, избавляющая от дубляжа
         """
 
-        if (type(dfs)== pd.core.series.Series) | (type(dfs) == pd.core.frame.DataFrame):
+        if (type(dfs) == pd.core.series.Series) | (type(dfs) == pd.core.frame.DataFrame):
             df = dfs.copy() if type(dfs) == pd.core.frame.DataFrame else pd.DataFrame(dfs)
             self.columns = df.columns
             self.index = df.index
             if self._init_preproc:
-                new_df = pd.DataFrame(self.preproc.fit_transform(df),index=df.index,columns=df.columns)
+                new_df = pd.DataFrame(self.preproc.fit_transform(df), index=df.index, columns=df.columns)
                 self._init_preproc = False
             else:
-                new_df = pd.DataFrame(self.preproc.transform(df),index=df.index,columns=df.columns)
-            assert len_seq + points_ahead + gap -1 <= len(df)                       
-            X_train, X_test, y_train, y_test= src.ts_train_test_split(df = new_df,
-                                                                      len_seq=len_seq,
-                                                                      points_ahead=points_ahead,
-                                                                      gap=gap,
-                                                                      shag=shag,
-                                                                      intersection=intersection,
-                                                                      test_size=test_size,
-                                                                      train_size=train_size,
-                                                                      random_state=random_state,
-                                                                      shuffle=False, # потому что потом нужно в основном итераторе
-                                                                      stratify=stratify)
+                new_df = pd.DataFrame(self.preproc.transform(df), index=df.index, columns=df.columns)
+            assert len_seq + points_ahead + gap - 1 <= len(df)
+            X_train, X_test, y_train, y_test = src.ts_train_test_split(df=new_df,
+                                                                       len_seq=len_seq,
+                                                                       points_ahead=points_ahead,
+                                                                       gap=gap,
+                                                                       shag=shag,
+                                                                       intersection=intersection,
+                                                                       test_size=test_size,
+                                                                       train_size=train_size,
+                                                                       random_state=random_state,
+                                                                       shuffle=False,
+                                                                       # потому что потом нужно в основном итераторе
+                                                                       stratify=stratify)
 
         elif type(dfs) == type(list()):
             # уже все pd.DataFrame
-            _df = pd.concat(dfs,ignore_index=True)
+            _df = pd.concat(dfs, ignore_index=True)
             if self._init_preproc:
                 self.preproc.fit(_df)
-                self._init_preproc = False            
+                self._init_preproc = False
             self.columns = _df.columns
-            self.index  =  _df.index
+            self.index = _df.index
 
-            X_train, X_test, y_train, y_test = [],[],[],[]
+            X_train, X_test, y_train, y_test = [], [], [], []
             _k = 0
             for df in dfs:
-                if ((type(df) == pd.core.series.Series) | (type(df) == pd.core.frame.DataFrame))==False:
-                    raise NameError('Type of dfs is unsupported') 
+                if ((type(df) == pd.core.series.Series) | (type(df) == pd.core.frame.DataFrame)) == False:
+                    raise NameError('Type of dfs is unsupported')
                 if not (len_seq + points_ahead + gap + 1 <= len(df)):
-                     _k+=1
-                     continue
-                
-                new_df = pd.DataFrame(self.preproc.transform(df),index=df.index,columns=df.columns)
-                _X_train, _X_test, _y_train, _y_test= src.ts_train_test_split(new_df,len_seq,
-                                                                          points_ahead=points_ahead,
-                                                                          gap=gap,
-                                                                          shag=shag,
-                                                                          intersection=intersection,
-                                                                          test_size=test_size,
-                                                                          train_size=train_size,
-                                                                          random_state=random_state,
-                                                                          shuffle=False,
-                                                                          stratify=stratify)
+                    _k += 1
+                    continue
+
+                new_df = pd.DataFrame(self.preproc.transform(df), index=df.index, columns=df.columns)
+                _X_train, _X_test, _y_train, _y_test = src.ts_train_test_split(new_df, len_seq,
+                                                                               points_ahead=points_ahead,
+                                                                               gap=gap,
+                                                                               shag=shag,
+                                                                               intersection=intersection,
+                                                                               test_size=test_size,
+                                                                               train_size=train_size,
+                                                                               random_state=random_state,
+                                                                               shuffle=False,
+                                                                               stratify=stratify)
                 X_train += _X_train
                 X_test += _X_test
                 y_train += _y_train
                 y_test += _y_test
-            
-            print(f'Пропущено {_k} датастов, из-за того что saples слишком малов в датасете. (len_seq + points_ahead + gap -1 <= len(df))')
+
+            print(
+                f'Пропущено {_k} датастов, из-за того что saples слишком малов в датасете. (len_seq + points_ahead + gap -1 <= len(df))')
 
         else:
-            raise NameError('Type of dfs is unsupported')  
-            
+            raise NameError('Type of dfs is unsupported')
+
         return X_train, X_test, y_train, y_test
-    
-    
-    
-    
+
     def _get_anomaly_timestamps(self,
                                 dfs,
                                 Loader,
@@ -152,68 +151,65 @@ class DL_AD():
                                 gap,
                                 shag,
                                 intersection,
-                                test_size,  
+                                test_size,
                                 random_state,
                                 shuffle,
                                 stratify,
+                                device,
                                 point_ahead_for_residuals=0):
         """
         Вспомогательная функция для  генерации всего
         """
-        X, _ , y_true, _  = self._get_Train_Test_sets(dfs=dfs,
-                                                      len_seq=len_seq,
-                                                      points_ahead=points_ahead, # 1 это default, так с остатками лучше не шутить до сих пор 
-                                                      gap=gap,
-                                                      shag=shag,
-                                                      intersection=intersection,
-                                                      test_size=test_size,  
-                                                      train_size=None,
-                                                      random_state=random_state,
-                                                      shuffle=shuffle,
-                                                      stratify=stratify)
+        X, _, y_true, _ = self._get_Train_Test_sets(dfs=dfs,
+                                                    len_seq=len_seq,
+                                                    points_ahead=points_ahead,
+                                                    # 1 это default, так с остатками лучше не шутить до сих пор
+                                                    gap=gap,
+                                                    shag=shag,
+                                                    intersection=intersection,
+                                                    test_size=test_size,
+                                                    train_size=None,
+                                                    random_state=random_state,
+                                                    shuffle=shuffle,
+                                                    stratify=stratify)
 
-
-        all_data_iterator = Loader(X,y_true, self.batch_size,shuffle=False)          
-        y_pred = self.model.run_epoch(all_data_iterator, None, None, phase='forecast', points_ahead=points_ahead)     
-        residuals = self.generate_res_func(y_pred,np.array(y_true))           
-        point_ahead_for_residuals = 0 # мы иногда прогнозим на 10 точек вперед, ну интересует все равно на одну точку впреред 
-        res_indices = [y_true[i].index[point_ahead_for_residuals] for i in range(len(y_true))]                                                
-        df_residuals = pd.DataFrame(residuals[:,point_ahead_for_residuals,:],columns=self.columns,index=res_indices).sort_index()
+        all_data_iterator = Loader(X, y_true, self.batch_size, shuffle=False)
+        y_pred = self.model.run_epoch(all_data_iterator, None, None, phase='forecast', points_ahead=points_ahead,
+                                      device=device)
+        residuals = self.generate_res_func(y_pred, np.array(y_true))
+        point_ahead_for_residuals = 0  # мы иногда прогнозим на 10 точек вперед, ну интересует все равно на одну точку впреред
+        res_indices = [y_true[i].index[point_ahead_for_residuals] for i in range(len(y_true))]
+        df_residuals = pd.DataFrame(residuals[:, point_ahead_for_residuals, :], columns=self.columns,
+                                    index=res_indices).sort_index()
         return df_residuals
-        
-        
 
-        
-# -----------------------------------------------------------------------------------------
-#     Формирование сутевой части класса 
-# -----------------------------------------------------------------------------------------    
-    
-    
-    def __init__(self,     
+    # -----------------------------------------------------------------------------------------
+    #     Формирование сутевой части класса
+    # -----------------------------------------------------------------------------------------
+
+    def __init__(self,
                  preproc=None,
                  generate_res_func=None,
                  res_analys_alg=None,
-                 
-                ):
-       
-        
+
+                 ):
+
         self.preproc = MinMaxScaler() if preproc is None else preproc
         self.generate_res_func = generate_residuals.abs if generate_res_func is None else generate_res_func
         self.res_analys_alg = stastics.Hotelling() if res_analys_alg is None else res_analys_alg
-        
 
-        
     def fit(self,
             dfs,
             model=None,
-            encod_decode_model = False,  # ужас, нужно это править, особенность encod_decode модели. Попытаться вообще еубрать эту переменную          
+            encod_decode_model=False,
+            # ужас, нужно это править, особенность encod_decode модели. Попытаться вообще еубрать эту переменную
             criterion=None,
             optimiser=None,
-            batch_size = 64,
-            len_seq = 10,
-            points_ahead = 5,
-            n_epochs = 100,
-            gap=0, 
+            batch_size=64,
+            len_seq=10,
+            points_ahead=5,
+            n_epochs=100,
+            gap=0,
             shag=1,
             intersection=True,
             test_size=0.2,
@@ -222,11 +218,11 @@ class DL_AD():
             shuffle=False,
             show_progress=True,
             show_figures=True,
-            best_model_file = './best_model.pt',
+            best_model_file='./best_model.pt',
             stratify=None,
             Loader=None,
-            
-           ):
+
+            ):
 
         """
         Обучение модели как для задачи прогнозирования так и для задачи anomaly
@@ -331,69 +327,68 @@ class DL_AD():
         ----------
         list of pd.datetime anomalies on initial dataset
         """
-        self._init_preproc=True # это кастыль для _get_Train_Test_sets
+        self._init_preproc = True  # это кастыль для _get_Train_Test_sets
         self.points_ahead = points_ahead
         self.len_seq = len_seq
         self.batch_size = batch_size
         dfs = dfs.copy()
         self.best_model_file = best_model_file
         self.encod_decode_model = encod_decode_model
-        
-         
+        if show_progress:
+            show_progress_text = ""
 
-# -----------------------------------------------------------------------------------------
-#     Формирование train_iterator и val_iteraror
-# -----------------------------------------------------------------------------------------
+        # -----------------------------------------------------------------------------------------
+        #     Формирование train_iterator и val_iteraror
+        # -----------------------------------------------------------------------------------------
         if Loader is None:
             Loader = src.Loader
-            
-        
+
         X_train, X_test, y_train, y_test = self._get_Train_Test_sets(dfs=dfs,
-                                                               len_seq=len_seq,
-                                                               points_ahead=points_ahead,
-                                                               gap=gap,
-                                                               shag=shag,
-                                                               intersection=intersection,
-                                                               test_size=test_size,
-                                                               train_size=train_size,
-                                                               random_state=random_state,
-                                                               shuffle=shuffle,  
-                                                               stratify=stratify) 
+                                                                     len_seq=len_seq,
+                                                                     points_ahead=points_ahead,
+                                                                     gap=gap,
+                                                                     shag=shag,
+                                                                     intersection=intersection,
+                                                                     test_size=test_size,
+                                                                     train_size=train_size,
+                                                                     random_state=random_state,
+                                                                     shuffle=shuffle,
+                                                                     stratify=stratify)
 
-        train_iterator = Loader(X_train, y_train,batch_size,shuffle=shuffle)
-        val_iterator   = Loader(X_test, y_test,batch_size,shuffle=shuffle)
+        train_iterator = Loader(X_train, y_train, batch_size, shuffle=shuffle)
+        val_iterator = Loader(X_test, y_test, batch_size, shuffle=shuffle)
 
-# -----------------------------------------------------------------------------------------
-#     Обучение моделей
-# -----------------------------------------------------------------------------------------
-            
+        # -----------------------------------------------------------------------------------------
+        #     Обучение моделей
+        # -----------------------------------------------------------------------------------------
 
         device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")
-    
-        
+
         if criterion is None:
             criterion = nn.MSELoss()
-            
+
         if model is None:
-            model = models.SimpleLSTM(len(self.columns),len(self.columns),device=device, seed=random_state)
+            model = models.SimpleLSTM(len(self.columns), len(self.columns), seed=random_state)
         self.model = model
-            
-            
+
         if optimiser is None:
             optimiser = torch.optim.Adam
             optimiser = optimiser(self.model.parameters())
         else:
             args = optimiser[1]
-            args['params']=self.model.parameters()
+            args['params'] = self.model.parameters()
             optimiser = optimiser[0](**args)
-            
-            
+
         history_train = []
         history_val = []
         best_val_loss = float('+inf')
-        for epoch in range(n_epochs):    
-            train_loss = self.model.run_epoch(train_iterator, optimiser, criterion, phase='train', points_ahead=points_ahead,encod_decode_model=self.encod_decode_model)#, writer=writer)
-            val_loss = self.model.run_epoch(val_iterator, None, criterion, phase='val', points_ahead=points_ahead,encod_decode_model=self.encod_decode_model)#, writer=writer)
+        for epoch in range(n_epochs):
+            train_loss = self.model.run_epoch(train_iterator, optimiser, criterion, phase='train',
+                                              points_ahead=points_ahead, encod_decode_model=self.encod_decode_model,
+                                              device=device)  # , writer=writer)
+            val_loss = self.model.run_epoch(val_iterator, None, criterion, phase='val', points_ahead=points_ahead,
+                                            encod_decode_model=self.encod_decode_model,
+                                            device=device)  # , writer=writer)
 
             history_train.append(train_loss)
             history_val.append(val_loss)
@@ -401,41 +396,49 @@ class DL_AD():
             if val_loss < best_val_loss:
                 best_val_loss = val_loss
                 torch.save(self.model.state_dict(), self.best_model_file)
-            
-            if show_progress:
-                print(f'Epoch: {epoch+1:02}')
-                print(f'\tTrain Loss: {train_loss:.3f}')
-                print(f'\t Val. Loss: {val_loss:.3f} ') 
 
-                
+            if show_figures:
+                display.clear_output(wait=True)
+                plt.figure()
+                plt.plot(history_train, label='Train')
+                plt.plot(history_val, label='Val')
+                plt.xlabel('Epoch')
+                plt.ylabel('MSE')
+                plt.legend()
+                plt.show()
+
+            if show_progress:
+                show_progress_text += f'Epoch: {epoch + 1:02} \n'
+                show_progress_text += f'\tTrain Loss: {train_loss:.3f} \n'
+                show_progress_text += f'\t Val. Loss: {val_loss:.3f} \n\n'
+                print(show_progress_text)
+
+
+
+
         self.model.load_state_dict(torch.load(self.best_model_file))
-        
+
         if show_progress:
+            print("After choosing the best model:")
             try:
-                test_iterator  = Loader(X_test, y_test, len(X_test),shuffle=False)
-                test_loss = self.model.run_epoch(test_iterator, None, criterion, phase='val',encod_decode_model=self.encod_decode_model)
+                test_iterator = Loader(X_test, y_test, len(X_test), shuffle=False)
+                test_loss = self.model.run_epoch(test_iterator, None, criterion, phase='val',
+                                                 encod_decode_model=self.encod_decode_model, device=device)
                 print(f'Test Loss: {test_loss:.3f}')
             except:
                 print('Весь X_test не помещается в память, тестим усреднением по батчам')
-                test_iterator  = Loader(X_test, y_test,batch_size,shuffle=False)
+                test_iterator = Loader(X_test, y_test, batch_size, shuffle=False)
                 test_loss = []
-                for epoch in range(n_epochs):  
-                    test_loss.append(self.model.run_epoch(test_iterator, None, criterion, phase='val',encod_decode_model=self.encod_decode_model))
+                for epoch in range(n_epochs):
+                    test_loss.append(self.model.run_epoch(test_iterator, None, criterion, phase='val',
+                                                          encod_decode_model=self.encod_decode_model, device=device))
                 print(f'Test Loss: {np.mean(test_loss):.3f}')
-            
-        if show_figures:
-            plt.figure()
-            plt.plot(history_train,label='Train')
-            plt.plot(history_val,label='Val')
-            plt.xlabel('Epoch')
-            plt.ylabel('MSE')
-            plt.legend()
-            plt.show()
 
-# -----------------------------------------------------------------------------------------
-#     Генерация остатков
-# -----------------------------------------------------------------------------------------            
-        df_residuals =  self._get_anomaly_timestamps(dfs=dfs,
+
+        # -----------------------------------------------------------------------------------------
+        #     Генерация остатков
+        # -----------------------------------------------------------------------------------------
+        df_residuals = self._get_anomaly_timestamps(dfs=dfs,
                                                     Loader=Loader,
                                                     len_seq=len_seq,
                                                     points_ahead=1,
@@ -444,26 +447,25 @@ class DL_AD():
                                                     intersection=intersection,
                                                     test_size=0,
                                                     random_state=None,
-                                                    shuffle=False,  
+                                                    shuffle=False,
                                                     stratify=stratify,
-                                                    point_ahead_for_residuals=0)        
-        self.anomaly_timestamps = self.res_analys_alg.fit_predict(df_residuals,show_figure=show_figures)
+                                                    device=device,
+                                                    point_ahead_for_residuals=0)
+        self.anomaly_timestamps = self.res_analys_alg.fit_predict(df_residuals, show_figure=show_figures)
         self.statistic = self.res_analys_alg.statistic
         self.ucl = self.res_analys_alg.ucl
         self.lcl = self.res_analys_alg.lcl
         return self.anomaly_timestamps
-        
-       
-    
-# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++  
-# xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
-# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++   
 
-   # накосячил тут с прогнозом на одну точку вперед. Могут быть проблемы если ahead !=1
+    # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+    # xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
+    # ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+
+    # накосячил тут с прогнозом на одну точку вперед. Могут быть проблемы если ahead !=1
     def predict_anomaly(self,
                         dfs,
                         Loader=None,
-                        gap=0, 
+                        gap=0,
                         shag=1,
                         intersection=True,
                         train_size=None,
@@ -472,9 +474,8 @@ class DL_AD():
                         stratify=None,
                         show_progress=True,
                         show_figures=True
-           ):
-        
-        
+                        ):
+
         """
         Поиск аномалий в новом наборе данных
         
@@ -493,16 +494,16 @@ class DL_AD():
         
         """
         len_seq = self.len_seq
-        
 
         if Loader is None:
             Loader = src.Loader
 
+        device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")
 
-# -----------------------------------------------------------------------------------------
-#     Генерация остатков
-# -----------------------------------------------------------------------------------------            
-        df_residuals =  self._get_anomaly_timestamps(dfs=dfs,
+        # -----------------------------------------------------------------------------------------
+        #     Генерация остатков
+        # -----------------------------------------------------------------------------------------
+        df_residuals = self._get_anomaly_timestamps(dfs=dfs,
                                                     Loader=Loader,
                                                     len_seq=len_seq,
                                                     points_ahead=1,
@@ -511,16 +512,15 @@ class DL_AD():
                                                     intersection=intersection,
                                                     test_size=0,
                                                     random_state=None,
-                                                    shuffle=False,  
+                                                    shuffle=False,
                                                     stratify=stratify,
-                                                    point_ahead_for_residuals=0)                                                                   
-        self.anomaly_timestamps = self.res_analys_alg.predict(df_residuals,show_figure=show_figures)
+                                                    device=device,
+                                                    point_ahead_for_residuals=0)
+        self.anomaly_timestamps = self.res_analys_alg.predict(df_residuals, show_figure=show_figures)
         self.statistic = self.res_analys_alg.statistic
         return self.anomaly_timestamps
 
-
-    
-    def forecast(self,df,points_ahead=None,Loader=None, show_figures=True):
+    def forecast(self, df, points_ahead=None, Loader=None, show_figures=True):
         """
         Прогнозирование временного ряда, в том числе векторного.
         
@@ -550,41 +550,76 @@ class DL_AD():
         ----------
         
         """
+        device = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")
+
         if Loader is None:
             Loader = src.Loader
-        
-        df  = df.copy()   
+
+        df = df.copy()
         points_ahead = points_ahead if points_ahead is not None else self.points_ahead
         len_seq = self.len_seq
-        batch_size = self.batch_size         
+        batch_size = self.batch_size
 
-        assert (type(df)== pd.core.series.Series) | (type(df) == pd.core.frame.DataFrame)
+        assert (type(df) == pd.core.series.Series) | (type(df) == pd.core.frame.DataFrame)
         df = df.copy() if type(df) == pd.core.frame.DataFrame else pd.DataFrame(df)
         df = df[-len_seq:]
         assert not self._init_preproc
         preproc_values = self.preproc.transform(df)
-        
-        
-        iterator = Loader(np.expand_dims(preproc_values,0),np.expand_dims(preproc_values,0), #ничего страшного, 'y' все равно не используется
-                          batch_size,shuffle=False)
 
-        y_pred = self.model.run_epoch(iterator, None, None, phase='forecast', points_ahead=points_ahead)[0]
+        iterator = Loader(np.expand_dims(preproc_values, 0), np.expand_dims(preproc_values, 0),
+                          # ничего страшного, 'y' все равно не используется
+                          batch_size, shuffle=False)
+
+        y_pred = self.model.run_epoch(iterator, None, None, phase='forecast', points_ahead=points_ahead, device=device)[
+            0]
         y_pred = self.preproc.inverse_transform(y_pred)
-        
+
         t_last = np.datetime64(df.index[-1])
-        delta_dime = np.timedelta64(df.index[-1] - df.index[-2]) 
-        new_index = pd.to_datetime(t_last + np.arange(1,points_ahead+1)*delta_dime)
-        y_pred = pd.DataFrame(y_pred,index=new_index,columns=df.columns)
-        
+        delta_dime = np.timedelta64(df.index[-1] - df.index[-2])
+        new_index = pd.to_datetime(t_last + np.arange(1, points_ahead + 1) * delta_dime)
+        y_pred = pd.DataFrame(y_pred, index=new_index, columns=df.columns)
+
         if show_figures:
-            pd.concat([df,y_pred])[-3*points_ahead:].plot()
-            plt.axvspan(t_last,y_pred.index[-1], alpha=0.2, color='green',label='forecast')
+            pd.concat([df, y_pred])[-3 * points_ahead:].plot()
+            plt.axvspan(t_last, y_pred.index[-1], alpha=0.2, color='green', label='forecast')
             plt.xlabel('Datetime')
             plt.ylabel('Value')
             plt.legend()
             plt.show()
-        
+
         return y_pred
 
-    
- 
\ No newline at end of file
+    def save(self, path='./pipeline.pcl'):
+        """
+        Method for saving pipeline.
+        It may be required for example after training.
+        CPU.
+        
+        Parameters
+        ----------
+            path : str
+        Путь до файла, для сохранения пайплайна. 
+        Пайлайн сохраняется в формате pickle
+        """
+
+        self.model.run_epoch(src.Loader(torch.zeros((1, self.len_seq, self.model.in_features), dtype=float),
+                                        torch.zeros((1, self.len_seq, self.model.in_features), dtype=float),
+                                        batch_size=1),
+                             None, None, phase='forecast', points_ahead=1, device=torch.device("cpu"))
+        with open(path, 'wb') as f:
+            pickle.dump(self, f)
+
+    def load(self, path='./pipeline.pcl'):
+        """
+        Method for loading pipeline.
+        It may be required for example after training.
+        
+        Parameters
+        ----------
+            path : str
+        Путь до сохраненного файла пайплайна. 
+        Пайлайн должен быть в формате pickle
+        """
+        with open(path, 'rb') as f:
+            pipeline = pickle.load(f)
+        self.__dict__.update(pipeline.__dict__)
